@@ -31,9 +31,13 @@
 #include "panel-menu-button.h"
 #include "panel-globals.h"
 #include "panel-properties-dialog.h"
+#include "panel-lockdown.h"
 
 #define SMALL_ICON_SIZE 20
 
+static GtkWidget *panel_applet_create_menu (AppletInfo  *info,
+					    PanelWidget *panel_widget);
+
 static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
@@ -81,10 +85,6 @@ panel_applet_toggle_locked (AppletInfo *info)
 	
 	locked = panel_widget_toggle_applet_locked (panel_widget, info->widget);
 
-	if (panel_toplevel_get_locked_down (panel_widget->toplevel) ||
-	    panel_profile_get_locked_down ())
-		return TRUE;
-
 	panel_applet_save_position (info, info->id, TRUE);
 	panel_applet_set_dnd_enabled (info, !locked);
 
@@ -104,8 +104,6 @@ panel_applet_lock (GtkWidget  *menuitem,
 
 	if (info->move_item)
 		gtk_widget_set_sensitive (info->move_item, !locked);
-	if (info->remove_item)
-		gtk_widget_set_sensitive (info->remove_item, !locked);
 }
 
 static void
@@ -144,6 +142,26 @@ panel_applet_clean (AppletInfo *info)
 	}
 }
 
+static void
+panel_applet_recreate_menu (AppletInfo	*info)
+{
+	GList *l;
+
+	if (!info->menu)
+		return;
+
+	for (l = info->user_menu; l; l = l->next) {
+		AppletUserMenu *menu = l->data;
+
+		menu->menuitem =NULL;
+		menu->submenu =NULL;
+	}
+
+	gtk_widget_unref (info->menu);
+	info->menu = panel_applet_create_menu (info,
+					       PANEL_WIDGET (info->widget->parent));
+}
+
 static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
@@ -187,7 +205,7 @@ applet_callback_callback (GtkWidget      *widget,
 
 			panel_properties_dialog_present (drawer->toplevel);
 		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help (screen, "wgospanel.xml", "gospanel-18");
+			panel_show_help (screen, "user-guide.xml", "gospanel-18");
 		}
 		break;
 	case PANEL_OBJECT_MENU:
@@ -243,80 +261,31 @@ panel_applet_get_callback (GList      *user_menu,
 }
 
 void
-panel_applet_add_callback (AppletInfo *info,
-			   const char *callback_name,
-			   const char *stock_item,
-			   const char *menuitem_text)
+panel_applet_add_callback (AppletInfo          *info,
+			   const char          *callback_name,
+			   const char          *stock_item,
+			   const char          *menuitem_text,
+			   CallbackEnabledFunc  is_enabled_func)
 {
 	AppletUserMenu *menu;
 
 	g_return_if_fail (info != NULL);
+	g_return_if_fail (panel_applet_get_callback (info->user_menu,
+						     callback_name) == NULL);
 
-	menu = panel_applet_get_callback (info->user_menu, callback_name);
-	if (menu == NULL) {
-		menu = g_new0 (AppletUserMenu, 1);
-		menu->name = g_strdup (callback_name);
-		menu->stock_item = g_strdup (stock_item);
-		menu->text = g_strdup (menuitem_text);
-		menu->sensitive = TRUE;
-		menu->info = info;
-		menu->menuitem = NULL;
-		menu->submenu = NULL;
-		info->user_menu = g_list_append (info->user_menu, menu);
-	} else {
-		g_free (menu->stock_item);
-		menu->stock_item = NULL;
-		g_free (menu->text);
-		menu->text = NULL;
-		menu->text = g_strdup (menuitem_text);
-		menu->stock_item = g_strdup (stock_item);
-	}
+	menu                  = g_new0 (AppletUserMenu, 1);
+	menu->name            = g_strdup (callback_name);
+	menu->stock_item      = g_strdup (stock_item);
+	menu->text            = g_strdup (menuitem_text);
+	menu->is_enabled_func = is_enabled_func;
+	menu->sensitive       = TRUE;
+	menu->info            = info;
+	menu->menuitem        = NULL;
+	menu->submenu         = NULL;
 
-	/*make sure the menu is rebuilt*/
-	if(info->menu) {
-		GList *list;
-		for(list=info->user_menu;list!=NULL;list=g_list_next(list)) {
-			AppletUserMenu *menu = list->data;
-			menu->menuitem=NULL;
-			menu->submenu=NULL;
-		}
-		gtk_widget_unref(info->menu);
-		info->menu = NULL;
-	}
-}
+	info->user_menu = g_list_append (info->user_menu, menu);
 
-void
-panel_applet_remove_callback (AppletInfo *info,
-			      const char *callback_name)
-{
-	AppletUserMenu *menu;
-
-	g_return_if_fail (info != NULL);
-	
-	menu = panel_applet_get_callback (info->user_menu, callback_name);
-	if (menu != NULL) {
-		info->user_menu = g_list_remove (info->user_menu, menu);
-		g_free(menu->name);
-		menu->name = NULL;
-		g_free(menu->stock_item);
-		menu->stock_item = NULL;
-		g_free(menu->text);
-		menu->text = NULL;
-		g_free(menu);
-	} else
-		return; /*it just isn't there*/
-
-	/*make sure the menu is rebuilt*/
-	if (info->menu != NULL) {
-		GList *list;
-		for (list = info->user_menu; list != NULL; list = list->next) {
-			AppletUserMenu *menu = list->data;
-			menu->menuitem = NULL;
-			menu->submenu = NULL;
-		}
-		gtk_widget_unref (info->menu);
-		info->menu = NULL;
-	}
+	panel_applet_recreate_menu (info);
 }
 
 static void
@@ -449,7 +418,10 @@ panel_applet_create_menu (AppletInfo  *info,
 			  G_CALLBACK (applet_menu_deactivate), info);
 
 	for (l = info->user_menu; l; l = l->next) {
-		AppletUserMenu *user_menu = (AppletUserMenu *)l->data;
+		AppletUserMenu *user_menu = l->data;
+
+		if (user_menu->is_enabled_func && !user_menu->is_enabled_func ())
+			continue;
 
 		add_to_submenus (info, "", user_menu->name, user_menu, 
 				 menu, info->user_menu);
@@ -457,8 +429,7 @@ panel_applet_create_menu (AppletInfo  *info,
 		added_anything = TRUE;
 	}
 
-	if ( ! panel_profile_get_locked_down () &&
-	     ! panel_toplevel_get_locked_down (panel_widget->toplevel)) {
+	if (!panel_lockdown_get_locked_down ()) {
 		GtkWidget *image;
 		gboolean   locked;
 		gboolean   lockable;
@@ -484,13 +455,7 @@ panel_applet_create_menu (AppletInfo  *info,
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_set_sensitive (menuitem, !locked && removable);
-
-		g_assert (info->remove_item == NULL);
-
-		info->remove_item = menuitem;
-		g_object_add_weak_pointer (G_OBJECT (menuitem),
-					   (gpointer *) &info->remove_item);
+		gtk_widget_set_sensitive (menuitem, removable);
 
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
@@ -654,8 +619,11 @@ panel_applet_destroy (GtkWidget  *widget,
 		}
 	}
 
+	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
+				      info);
+
 	if (info->menu)
-		gtk_widget_unref (info->menu);
+		g_object_unref (info->menu);
 	info->menu = NULL;
 
 	if (info->data_destroy)
@@ -752,8 +720,12 @@ panel_applet_load_idle_handler (gpointer dummy)
 
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 
-	if (applet->right_stick && !panel_widget->packed)
-		applet->position = panel_widget->size - applet->position;
+	if (applet->right_stick) {
+		if (!panel_widget->packed)
+			applet->position = panel_widget->size - applet->position;
+		else
+			applet->position = -1;
+	}
 
 	switch (applet->type) {
 	case PANEL_OBJECT_BONOBO:
@@ -853,6 +825,8 @@ panel_applet_compare (const PanelAppletToLoad *a,
 
 	if ((c = strcmp (a->toplevel_id, b->toplevel_id)))
 		return c;
+	else if (a->right_stick != b->right_stick)
+		return b->right_stick ? -1 : 1;
 	else
 		return a->position - b->position;
 }
@@ -1059,7 +1033,6 @@ panel_applet_register (GtkWidget       *applet,
 		       const char      *id)
 {
 	AppletInfo *info;
-	gboolean    insert_at_pos = FALSE;
 	
 	g_return_val_if_fail (applet != NULL && panel != NULL, NULL);
 
@@ -1077,11 +1050,13 @@ panel_applet_register (GtkWidget       *applet,
 	info->data_destroy = data_destroy;
 	info->user_menu    = NULL;
 	info->move_item    = NULL;
-	info->remove_item  = NULL;
 	info->id           = g_strdup (id);
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
+	panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
+				   info);
+
 	if (type == PANEL_OBJECT_DRAWER) {
 		Drawer *drawer = data;
 		PanelWidget *assoc_panel;
@@ -1100,24 +1075,22 @@ panel_applet_register (GtkWidget       *applet,
 
 	registered_applets = g_slist_append (registered_applets, info);
 
-	/* if exact pos is on then insert at that precise location */
-	if (exactpos)
-		insert_at_pos = TRUE;
-
-	if (panel_widget_add (panel, applet, locked, pos, insert_at_pos) == -1) {
+	if (panel_widget_add (panel, applet, locked, pos, exactpos) == -1 &&
+	    panel_widget_add (panel, applet, locked, 0, TRUE) == -1) {
 		GSList *l;
 
-		for (l = panels; l; l = l->next)
+		for (l = panels; l; l = l->next) {
+			panel = PANEL_WIDGET (l->data);
+
 			if (panel_widget_add (panel, applet, locked, 0, TRUE) != -1)
 				break;
+		}
 
 		if (!l) {
 			g_warning (_("Can't find an empty spot"));
 			panel_profile_delete_object (info);
 			return NULL;
 		}
-
-		panel = PANEL_WIDGET (l->data);
 	}
 
 	if (BUTTON_IS_WIDGET (applet) ||
@@ -1164,58 +1137,59 @@ panel_applet_get_position (AppletInfo *applet)
 	return applet_data->pos;
 }
 
-/* True if all the keys relevant to moving are writable
-   (position, toplevel_id, panel_right_stick) */
 gboolean
 panel_applet_can_freely_move (AppletInfo *applet)
 {
-	GConfClient *client;
-	const char *profile;
-	PanelGConfKeyType key_type;
-	const char *key;
-	PanelWidget *panel_widget;
+	PanelWidget       *panel_widget;
+	GConfClient       *client;
+	PanelGConfKeyType  key_type;
+	const char        *profile;
+	const char        *key;
+
+	if (panel_lockdown_get_locked_down ())
+		return FALSE;
 
 	panel_widget = PANEL_WIDGET (applet->widget->parent);
 
-	if (panel_toplevel_get_locked_down (panel_widget->toplevel) ||
-	    panel_profile_get_locked_down ())
-		return FALSE;
-
-	client = panel_gconf_get_client ();
+	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
-	key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	
+	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
        
 	key = panel_gconf_full_key (key_type, profile, applet->id, "position");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	key = panel_gconf_full_key (key_type, profile, applet->id, "toplevel_id");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	key = panel_gconf_full_key (key_type, profile, applet->id, "panel_right_stick");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	return TRUE;
 }
 
-/* True if the locked flag is writable */
 gboolean
 panel_applet_lockable (AppletInfo *applet)
 {
-	GConfClient *client;
-	const char *profile;
-	PanelGConfKeyType key_type;
-	const char *key;
-	PanelWidget *panel_widget;
+	GConfClient        *client;
+	PanelWidget        *panel_widget;
+	PanelGConfKeyType   key_type;
+	const char         *profile;
+	const char         *key;
 
+	if (panel_lockdown_get_locked_down ())
+		return FALSE;
+	
 	panel_widget = PANEL_WIDGET (applet->widget->parent);
 
-	if (panel_toplevel_get_locked_down (panel_widget->toplevel) ||
-	    panel_profile_get_locked_down ())
-		return FALSE;
-
-	client = panel_gconf_get_client ();
+	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
-	key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	
+	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+
 	key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
 
 	return gconf_client_key_is_writable (client, key, NULL);
