@@ -19,6 +19,7 @@ typedef struct {
 	GtkWidget       *widget;
 
 	GtkWidget       *menu;
+	GtkWidget       *move_item;
 	GList           *user_menu;
 
 	gpointer         data;
@@ -27,29 +28,37 @@ typedef struct {
 	char            *id;
 } AppletInfo;
 
+typedef gboolean (* CallbackEnabledFunc) (void);
+
 typedef struct {
-	gchar        *name;
-	gchar        *stock_item;
-	gchar        *text;
+	char                *name;
+	char                *stock_item;
+	char                *text;
 
-	gint          sensitive;
-	AppletInfo   *info;
+	CallbackEnabledFunc  is_enabled_func;
 
-	GtkWidget    *menuitem;
-	GtkWidget    *submenu;
+	int                  sensitive;
+	AppletInfo          *info;
+
+	GtkWidget           *menuitem;
+	GtkWidget           *submenu;
 } AppletUserMenu;
 
 AppletInfo *panel_applet_register    (GtkWidget       *applet,
 				      gpointer         data,
 				      GDestroyNotify   data_destroy,
 				      PanelWidget     *panel,
+				      gboolean         locked,
 				      gint             pos,
 				      gboolean         exactpos,
 				      PanelObjectType  type,
 				      const char      *id);
 
-const char *panel_applet_get_id       (AppletInfo    *info);
-AppletInfo *panel_applet_get_by_id    (const char    *id);
+const char *panel_applet_get_id           (AppletInfo      *info);
+const char *panel_applet_get_id_by_widget (GtkWidget       *widget);
+AppletInfo *panel_applet_get_by_id        (const char      *id);
+AppletInfo *panel_applet_get_by_type      (PanelObjectType  object_type);
+
 GSList     *panel_applet_list_applets (void);
 
 void        panel_applet_clean        (AppletInfo    *info);
@@ -58,17 +67,17 @@ void panel_applet_queue_applet_to_load (char            *id,
 					PanelObjectType  type,
 					char            *toplevel_id,
 					int              position,
-					gboolean         right_stick);
+					gboolean         right_stick,
+					gboolean         locked);
 void panel_applet_load_queued_applets  (void);
+gboolean panel_applet_on_load_queue    (const char *id);
 
 
-void            panel_applet_add_callback    (AppletInfo  *info,
-					      const gchar *callback_name,
-					      const gchar *stock_item,
-					      const gchar *menuitem_text);
-
-void            panel_applet_remove_callback (AppletInfo *info,
-					      const char *callback_name);
+void            panel_applet_add_callback    (AppletInfo          *info,
+					      const gchar         *callback_name,
+					      const gchar         *stock_item,
+					      const gchar         *menuitem_text,
+					      CallbackEnabledFunc  is_enabled_func);
 
 AppletUserMenu *panel_applet_get_callback    (GList       *user_menu,
 					      const gchar *name);
@@ -80,10 +89,19 @@ void        panel_applet_save_position           (AppletInfo *applet_info,
 
 int         panel_applet_get_position    (AppletInfo *applet);
 
+/* True if all the keys relevant to moving are writable
+   (position, toplevel_id, panel_right_stick) */
+gboolean    panel_applet_can_freely_move (AppletInfo *applet);
+
+/* True if the locked flag is writable */
+gboolean    panel_applet_lockable (AppletInfo *applet);
+
 void        panel_applet_menu_set_recurse (GtkMenu     *menu,
 					   const gchar *key,
 					   gpointer     data);
 
+gboolean    panel_applet_toggle_locked  (AppletInfo *info);
+
 G_END_DECLS
 
 #endif
