@@ -29,6 +29,7 @@
 #include "xstuff.h"
 #include "panel-globals.h"
 #include "panel-stock-icons.h"
+#include "panel-lockdown.h"
 
 static void
 drawer_click (GtkWidget *w, Drawer *drawer)
@@ -92,7 +93,7 @@ key_press_drawer (GtkWidget   *widget,
 	gboolean retval = TRUE;
 	GtkOrientation orient;
 
-	if (event->state == gtk_accelerator_get_default_mod_mask ())
+	if (event->state & gtk_accelerator_get_default_mod_mask ())
 		return FALSE;
 
 	orient = PANEL_WIDGET (drawer->button->parent)->orient;
@@ -168,7 +169,7 @@ key_press_drawer_widget (GtkWidget   *widget,
 
 	gtk_window_present (GTK_WINDOW (panel_widget->toplevel));
 
-	if (event->state == GDK_SHIFT_MASK ||
+	if ((event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_SHIFT_MASK ||
 	    panel_toplevel_get_is_hidden (drawer->toplevel))
 		return TRUE;
 
@@ -308,6 +309,20 @@ drag_data_get_cb (GtkWidget          *widget,
 	g_free (foo);
 }
 
+static void
+set_tooltip_and_name (Drawer     *drawer,
+		      const char *tooltip)
+{
+	g_return_if_fail (drawer != NULL);
+	g_return_if_fail (tooltip != NULL);
+
+	if (tooltip && !tooltip [0])
+		tooltip = NULL;
+
+	panel_toplevel_set_name (drawer->toplevel, tooltip ? tooltip : _("Drawer"));
+	gtk_tooltips_set_tip (panel_tooltips, drawer->button, tooltip, NULL);
+}
+
 static Drawer *
 create_drawer_applet (PanelToplevel    *toplevel,
 		      PanelToplevel    *parent_toplevel,
@@ -317,9 +332,12 @@ create_drawer_applet (PanelToplevel    *toplevel,
 		      PanelOrientation  orientation)
 {
 	Drawer *drawer;
+	AtkObject *atk_obj;
 	
 	drawer = g_new0 (Drawer, 1);
 
+	drawer->toplevel = toplevel;
+
 	if (!use_custom_icon || !custom_icon || !custom_icon [0]) {
 		drawer->button = button_widget_new_from_stock (PANEL_STOCK_DRAWER,
 							       TRUE, orientation);
@@ -334,12 +352,11 @@ create_drawer_applet (PanelToplevel    *toplevel,
 		g_free (drawer);
 		return NULL;
 	}
+	atk_obj = gtk_widget_get_accessible (drawer->button);
+	atk_object_set_name (atk_obj, _("Drawer")); 
+
+	set_tooltip_and_name (drawer, tooltip);
 
-	if (tooltip && tooltip [0]) {
-		panel_toplevel_set_name (toplevel, tooltip);
-		gtk_tooltips_set_tip (panel_tooltips, drawer->button, tooltip, NULL);
-	}
-	
 	gtk_drag_dest_set (drawer->button, 0, NULL, 0, 0); 
 
 	g_signal_connect (drawer->button, "drag_data_get",
@@ -364,8 +381,6 @@ create_drawer_applet (PanelToplevel    *toplevel,
 
 	gtk_widget_show (drawer->button);
 
-	drawer->toplevel = toplevel;
-
 	g_signal_connect (drawer->toplevel, "key_press_event",
 			  G_CALLBACK (key_press_drawer_widget), drawer);
 
@@ -492,10 +507,8 @@ panel_drawer_tooltip_changed (GConfClient *client,
 
 	tooltip = gconf_value_get_string (entry->value);
 
-	if (tooltip && tooltip [0]) {
-		panel_toplevel_set_name (drawer->toplevel, tooltip);
-		gtk_tooltips_set_tip (panel_tooltips, drawer->button, tooltip, NULL);
-	}
+	set_tooltip_and_name (drawer,
+			      gconf_value_get_string (entry->value));
 }
 
 static void
@@ -530,6 +543,12 @@ panel_drawer_connect_to_gconf (Drawer *drawer)
 	g_assert (i == PANEL_DRAWER_N_LISTENERS);
 }
 
+static gboolean
+drawer_properties_enabled (void)
+{
+  return !panel_lockdown_get_locked_down ();
+}
+
 static void
 load_drawer_applet (char          *toplevel_id,
 		    const char    *custom_icon,
@@ -554,13 +573,15 @@ load_drawer_applet (char          *toplevel_id,
 	if (!toplevel)
 		toplevel = create_drawer_toplevel (id);
 
-	if (toplevel) 
+	if (toplevel) {
+		panel_toplevel_hide (toplevel, FALSE, -1);
 		drawer = create_drawer_applet (toplevel,
 					       parent_toplevel,
 					       tooltip,
 					       custom_icon,
 					       use_custom_icon,
 					       orientation);
+	}
 
 	if (!drawer)
 		return;
@@ -585,16 +606,17 @@ load_drawer_applet (char          *toplevel_id,
 	panel_widget_set_applet_expandable (panel_widget, GTK_WIDGET (drawer->button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel_widget, GTK_WIDGET (drawer->button), TRUE);
 
-	gtk_window_present (GTK_WINDOW (drawer->toplevel));
+	panel_applet_add_callback (drawer->info,
+				   "properties",
+				   GTK_STOCK_PROPERTIES,
+				   _("_Properties"),
+				   drawer_properties_enabled);
 
-	if (!commie_mode)
-		panel_applet_add_callback (drawer->info,
-					   "properties",
-					   GTK_STOCK_PROPERTIES,
-					   _("_Properties"));
-
-	panel_applet_add_callback (
-		drawer->info, "help", GTK_STOCK_HELP, _("_Help"));
+	panel_applet_add_callback (drawer->info,
+				   "help",
+				   GTK_STOCK_HELP,
+				   _("_Help"),
+				   NULL);
 
 	panel_drawer_connect_to_gconf (drawer);
 }
@@ -759,3 +781,46 @@ panel_drawer_set_dnd_enabled (Drawer   *drawer,
 	} else
 		gtk_drag_source_unset (drawer->button);
 }
+
+static void
+drawer_deletion_response (GtkWidget     *dialog,
+                        int            response,
+                        Drawer         *drawer)
+{
+        if (response == GTK_RESPONSE_OK)
+		panel_profile_delete_object (drawer->info);
+
+        gtk_widget_destroy (dialog);
+}
+
+void 
+drawer_query_deletion (Drawer *drawer)
+{
+	GtkWidget *dialog;
+
+	 if (drawer->toplevel) {
+		PanelWidget *panel_widget;
+
+		panel_widget = panel_toplevel_get_panel_widget (
+							drawer->toplevel);
+
+		if (!panel_global_config_get_confirm_panel_remove () ||
+		    !g_list_length (panel_widget->applet_list)) {
+			panel_profile_delete_object (drawer->info);
+			return;
+		}
+
+		dialog = panel_deletion_dialog (drawer->toplevel);
+
+		g_signal_connect (dialog, "response",
+			  	  G_CALLBACK (drawer_deletion_response),
+			  	  drawer);
+
+		g_signal_connect_object (drawer->toplevel, "destroy",
+					 G_CALLBACK (gtk_widget_destroy),
+					 dialog,
+					 G_CONNECT_SWAPPED);
+
+        	gtk_widget_show_all (dialog);
+	}
+}
