@@ -38,6 +38,9 @@
 #include "applet.h"
 #include "panel-marshal.h"
 #include "panel-background.h"
+#include "panel-lockdown.h"
+
+#define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
 #undef PANEL_APPLET_FRAME_DEBUG
 
@@ -50,6 +53,7 @@
 #define PROPERTY_BACKGROUND "panel-applet-background"
 #define PROPERTY_FLAGS      "panel-applet-flags"
 #define PROPERTY_SIZE_HINTS "panel-applet-size-hints"
+#define PROPERTY_LOCKED_DOWN "panel-applet-locked-down"
 
 
 struct _PanelAppletFramePrivate {
@@ -78,13 +82,14 @@ typedef enum {
 	APPLET_FLAGS_NONE   = 0,
 	APPLET_EXPAND_MAJOR = 1 << 0,
 	APPLET_EXPAND_MINOR = 1 << 1,
-	APPLET_HAS_HANDLE   = 1 << 2,
+	APPLET_HAS_HANDLE   = 1 << 2
 } PanelAppletFlags;
 
 static void
 panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 {
 	PanelWidget *panel_widget;
+	gboolean     locked_down;
 	gboolean     locked;
 	gboolean     lockable;
 	gboolean     movable;
@@ -104,6 +109,7 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 				      locked ? _("Un_lock") : _("_Lock"),
 				      NULL);
 
+	/* First sensitivity */
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
 				      "/commands/LockAppletToPanel",
 				      "sensitive",
@@ -121,6 +127,27 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 				      "sensitive",
 				      locked ? "0" : (movable ? "1" : "0"),
 				      NULL);
+
+	/* Second visibility */
+	locked_down = panel_lockdown_get_locked_down ();
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/LockAppletToPanel",
+				      "hidden",
+				      locked_down ? "1" : "0",
+				      NULL);
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/RemoveAppletFromPanel",
+				      "hidden",
+				      locked_down ? "1" : "0",
+				      NULL);
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/MoveApplet",
+				      "hidden",
+				      locked_down ? "1" : "0",
+				      NULL);
 }
 
 static void
@@ -274,6 +301,9 @@ panel_applet_frame_load (const gchar *iid,
 	panel_applet_frame_set_info (frame, info);
 	panel_applet_frame_sync_menu_state (frame);
 	panel_applet_frame_init_properties (frame);
+
+	panel_lockdown_notify_add (G_CALLBACK (panel_applet_frame_sync_menu_state),
+				   frame);
 }
 
 void
@@ -364,6 +394,16 @@ void
 panel_applet_frame_change_size (PanelAppletFrame *frame,
 				PanelSize         size)
 {
+	/* Normalise the size to the constants defined in
+	 * the IDL.
+	 */
+	size = size <= PANEL_SIZE_XX_SMALL ? PANEL_SIZE_XX_SMALL :
+	       size <= PANEL_SIZE_X_SMALL  ? PANEL_SIZE_X_SMALL  :
+	       size <= PANEL_SIZE_SMALL    ? PANEL_SIZE_SMALL    :
+	       size <= PANEL_SIZE_MEDIUM   ? PANEL_SIZE_MEDIUM   :
+	       size <= PANEL_SIZE_LARGE    ? PANEL_SIZE_LARGE    :
+	       size <= PANEL_SIZE_X_LARGE  ? PANEL_SIZE_X_LARGE  : PANEL_SIZE_XX_LARGE;
+		 
 	bonobo_pbclient_set_short (frame->priv->property_bag, 
 				   PROPERTY_SIZE,
 				   size,
@@ -393,11 +433,13 @@ panel_applet_frame_change_background (PanelAppletFrame    *frame,
 	bg_str = panel_applet_frame_get_background_string (
 			frame, PANEL_WIDGET (GTK_WIDGET (frame)->parent), type);
 
-	bonobo_pbclient_set_string (frame->priv->property_bag,
-				    PROPERTY_BACKGROUND,
-				    bg_str, NULL);
+	if (bg_str != NULL) {
+		bonobo_pbclient_set_string (frame->priv->property_bag,
+					    PROPERTY_BACKGROUND,
+					    bg_str, NULL);
 
-	g_free (bg_str);
+		g_free (bg_str);
+	}
 }
 
 void
@@ -412,6 +454,9 @@ panel_applet_frame_finalize (GObject *object)
 {
 	PanelAppletFrame *frame = PANEL_APPLET_FRAME (object);
 
+	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_frame_sync_menu_state),
+				      frame);
+
 	if (frame->priv->property_bag)
 		bonobo_object_release_unref (
 			frame->priv->property_bag, NULL);
@@ -427,9 +472,6 @@ panel_applet_frame_finalize (GObject *object)
 	g_free (frame->priv->iid);
 	frame->priv->iid = NULL;
 
-        g_free (frame->priv);
-        frame->priv = NULL;
-
         parent_class->finalize (object);
 }
 
@@ -488,6 +530,28 @@ panel_applet_frame_expose (GtkWidget      *widget,
 	return FALSE;
 }
 
+static void
+panel_applet_frame_update_background_size (PanelAppletFrame *frame,
+					   GtkAllocation    *old_allocation,
+					   GtkAllocation    *new_allocation)
+{
+	PanelBackground *background;
+
+	if (old_allocation->x      == new_allocation->x &&
+	    old_allocation->y      == new_allocation->y &&
+	    old_allocation->width  == new_allocation->width &&
+	    old_allocation->height == new_allocation->height)
+		return;
+
+	background = &frame->priv->panel->background;
+
+	if (background->type == PANEL_BACK_NONE ||
+	   (background->type == PANEL_BACK_COLOR && !background->has_alpha))
+		return;
+
+	panel_applet_frame_change_background (frame, background->type);
+}
+
 static void
 panel_applet_frame_size_request (GtkWidget      *widget,
 				 GtkRequisition *requisition)
@@ -539,12 +603,22 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	PanelAppletFrame *frame;
 	GtkBin           *bin;
 	GtkAllocation     new_allocation;
+	GtkAllocation     old_allocation;
+
+	old_allocation.x      = widget->allocation.x;
+	old_allocation.y      = widget->allocation.y;
+	old_allocation.width  = widget->allocation.width;
+	old_allocation.height = widget->allocation.height;
 
 	frame = PANEL_APPLET_FRAME (widget);
 	bin = GTK_BIN (widget);
 
 	if (!frame->priv->has_handle) {
-		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget,
+								allocation);
+		panel_applet_frame_update_background_size (frame,
+							   &old_allocation,
+							   allocation);
 		return;
 	}
 
@@ -604,6 +678,10 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 		gtk_widget_size_allocate (bin->child, &new_allocation);
   
 	frame->priv->child_allocation = new_allocation;
+
+	panel_applet_frame_update_background_size (frame,
+						   &old_allocation,
+						   allocation);
 }
 
 static inline gboolean
@@ -699,6 +777,7 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 		char        *iid;
 		char        *id = NULL;
 		int          position = -1;
+		gboolean     locked = FALSE;
 
 		panel = frame->priv->panel;
 		iid   = g_strdup (frame->priv->iid);
@@ -706,11 +785,11 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 		if (info) {
 			id = g_strdup (info->id);
 			position  = panel_applet_get_position (info);
+			locked = panel_widget_get_applet_locked (panel, info->widget);
 			panel_applet_clean (info);
 		}
 
-		panel_applet_frame_load (iid, panel,
-					 panel_widget_get_applet_locked (panel, info->widget),
+		panel_applet_frame_load (iid, panel, locked,
 					 position, TRUE, id);
 
 		g_free (iid);
@@ -770,7 +849,9 @@ panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 	GtkWidget *dialog;
 	GdkScreen *screen;
 	char      *applet_name = NULL;
-	char      *txt;
+	char      *applet_txt;
+	char      *dialog_txt;
+	gboolean   locked_down;
 
 	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
 
@@ -779,30 +860,43 @@ panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 	if (frame->priv->iid)
 		applet_name = panel_applet_frame_get_name (frame->priv->iid);
 
-	txt = g_strdup_printf (
-			_("The %s applet appears to have died "
-			  "unexpectedly\n\n"
-			  "Reload this applet?\n\n"
-			  "(If you choose not to reload it at this time"
-			  " you can always add it by right clicking on "
-			  "the panel and clicking on the \"Add to Panel\""
-			  " submenu)"), applet_name ? applet_name : "");
+	locked_down = panel_lockdown_get_locked_down ();
+
+	applet_txt = g_strdup_printf (
+			_("The \"%s\" applet appears to have died "
+			  "unexpectedly.\n\n"
+			  "Do you want to reload this applet?"),
+			applet_name ? applet_name : "");
+	if (!locked_down) {
+		dialog_txt = g_strconcat(applet_txt,
+				_("\n\n"
+				  "(If you choose not to reload it at this time"
+				  " you can always add it by right clicking on "
+				  "the panel and clicking on the "
+				  "\"Add to Panel\" submenu)"),
+				NULL);
+		g_free (applet_txt);
+	} else {
+		dialog_txt = applet_txt;
+	}
 
 	dialog = gtk_message_dialog_new (
 				NULL,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_YES_NO,
-				txt);
+				dialog_txt);
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (panel_applet_frame_reload_response),
 			  g_object_ref (frame));
 
+	panel_widget_register_open_dialog (frame->priv->panel, dialog);
+
 	gtk_widget_show (dialog);
 	g_free (applet_name);
-	g_free (txt);
+	g_free (dialog_txt);
 }
 
 static inline void
@@ -832,6 +926,11 @@ register_stock_item (void)
         }
 }
 
+enum {
+	LOADING_FAILED_RESPONSE_DONT_DELETE,
+	LOADING_FAILED_RESPONSE_DELETE
+};
+
 static void
 panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 				   CORBA_Environment *ev,
@@ -840,29 +939,59 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 				   GtkWindow         *panel)
 {
 	GtkWidget *dialog;
-	char      *error;
 	int        response;
+	char      *problem_txt;
+	gboolean   locked_down;
 
-	error = bonobo_exception_get_text (ev);
+	locked_down = panel_lockdown_get_locked_down ();
 
-	dialog = gtk_message_dialog_new (
-				NULL, 0,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("The panel encountered a problem while loading \"%s\"\n"
-				  "Details: %s\n\n"
-				  "Do you want to delete the applet from your configuration?"),
-				iid, error);
+	problem_txt = g_strdup_printf (_("The panel encountered a problem "
+					 "while loading \"%s\"."), iid);
 
-	g_free (error);
+	if (ev != NULL) {
+		char *error = bonobo_exception_get_text (ev);
+		char *detail_txt = g_strdup_printf (_("Details: %s"), error);
+
+		dialog = gtk_message_dialog_new (
+					NULL, 0,
+					locked_down ? GTK_MESSAGE_INFO : GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					"%s\n%s%s",
+					problem_txt,
+					detail_txt,
+					locked_down ? "" :
+					_("\n\n"
+					  "Do you want to delete the applet "
+					  "from your configuration?"));
+		g_free (error);
+		g_free (detail_txt);
+	} else {
+		dialog = gtk_message_dialog_new (
+					NULL, 0,
+					locked_down ? GTK_MESSAGE_INFO : GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					"%s%s",
+					problem_txt,
+					locked_down ? "" :
+					_("\n\n"
+					  "Do you want to delete the applet "
+					  "from your configuration?"));
+	}
 
 	register_stock_item ();
 
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				PANEL_STOCK_DONT_DELETE, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
-				NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	if (locked_down) {
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+					GTK_STOCK_OK, LOADING_FAILED_RESPONSE_DONT_DELETE,
+					NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), LOADING_FAILED_RESPONSE_DONT_DELETE);
+	} else {
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+					PANEL_STOCK_DONT_DELETE, LOADING_FAILED_RESPONSE_DONT_DELETE,
+					GTK_STOCK_DELETE, LOADING_FAILED_RESPONSE_DELETE,
+					NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), LOADING_FAILED_RESPONSE_DELETE);
+	}
 
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_window_get_screen (panel));
@@ -871,13 +1000,10 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 
 	gtk_widget_destroy (dialog);
 
-	if (response == GTK_RESPONSE_OK) {
-		/* if we can't write to applets list we can't really delete
-		   it, so we'll just ignore this.  There's nothing we can
-		   do about it. */
-		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
-			panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
-	}
+	if (response == LOADING_FAILED_RESPONSE_DELETE &&
+	    !locked_down &&
+	    panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+		panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
 }
 
 static void
@@ -896,13 +1022,15 @@ panel_applet_frame_class_init (PanelAppletFrameClass *klass,
 	widget_class->size_allocate        = panel_applet_frame_size_allocate;
 	widget_class->button_press_event   = panel_applet_frame_button_changed;
 	widget_class->button_release_event = panel_applet_frame_button_changed;
+
+	g_type_class_add_private (klass, sizeof (PanelAppletFramePrivate));
 }
 
 static void
 panel_applet_frame_instance_init (PanelAppletFrame      *frame,
 				  PanelAppletFrameClass *klass)
 {
-	frame->priv = g_new0 (PanelAppletFramePrivate, 1);
+	frame->priv = PANEL_APPLET_FRAME_GET_PRIVATE (frame);
 
 	frame->priv->applet_shell     = CORBA_OBJECT_NIL;
 	frame->priv->property_bag     = CORBA_OBJECT_NIL;
@@ -1025,16 +1153,23 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 {
 	char *retval;
 	char *bg_str;
+	gboolean locked_down;
 
 	bg_str = panel_applet_frame_get_background_string (
 				frame, panel, panel->background.type);
 
+	if (bg_str == NULL)
+		bg_str = g_strdup ("");
+
+	locked_down = panel_lockdown_get_locked_down ();
+
 	retval = g_strdup_printf (
 			"%s!prefs_key=/apps/panel/profiles/%s/applets/%s/prefs;"
-			"background=%s;orient=%s;size=%s",
+			"background=%s;orient=%s;size=%s;locked_down=%s",
 			iid, panel_profile_get_name (), id, bg_str,
 			panel_applet_frame_get_orient_string (frame, panel),
-			panel_applet_frame_get_size_string (frame, panel));
+			panel_applet_frame_get_size_string (frame, panel),
+			locked_down ? "true" : "false");
 
 	g_free (bg_str);
 
@@ -1070,6 +1205,9 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	frame->priv->panel = panel;
 
+	if (panel_lockdown_is_applet_disabled (iid))
+		return NULL;
+
 	moniker = panel_applet_frame_construct_moniker (frame, panel, iid, id);
 
 	/* FIXME: this should really use bonobo_get_object_async */
@@ -1080,7 +1218,9 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	g_free (moniker);
 
-	if (BONOBO_EX (&ev)) {
+	/* according to the source of bonobo control == NULL && no
+	   exception can happen, so handle it */
+	if (BONOBO_EX (&ev) || control == CORBA_OBJECT_NIL) {
 		panel_applet_frame_loading_failed (
 			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
 		CORBA_exception_free (&ev);
@@ -1089,30 +1229,38 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	frame->priv->iid = g_strdup (iid);
 
-	cnx_status = ORBit_small_get_connection_status (control);
-	if (cnx_status != ORBIT_CONNECTION_IN_PROC)
-		g_signal_connect_object (
-			ORBit_small_get_connection (control),
-			"broken",
-			G_CALLBACK (panel_applet_frame_cnx_broken),
-			frame,
-			G_CONNECT_SWAPPED);
-	
-	CORBA_exception_free (&ev);
-
 	widget = bonobo_widget_new_control_from_objref (control, CORBA_OBJECT_NIL);
 
 	bonobo_object_release_unref (control, NULL);
 
 	if (!widget) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
 		g_warning (G_STRLOC ": failed to load applet %s", iid);
 		return NULL;
 	}
 
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
+	if (control_frame == NULL) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get control frame)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
 
 	frame->priv->property_bag = 
-		bonobo_control_frame_get_control_property_bag (control_frame, NULL);
+		bonobo_control_frame_get_control_property_bag (control_frame, &ev);
+	if (frame->priv->property_bag == NULL || BONOBO_EX (&ev)) {
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
+		CORBA_exception_free (&ev);
+		g_warning (G_STRLOC ": failed to load applet %s (can't get property bag)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
 
 	bonobo_event_source_client_add_listener (frame->priv->property_bag,
 						 (BonoboListenerCallbackFn) panel_applet_frame_event_listener,
@@ -1121,9 +1269,15 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 						 frame);
 	
 	frame->priv->ui_component =
-		bonobo_control_frame_get_popup_component (control_frame, NULL);
-	if (!frame->priv->ui_component)
+		bonobo_control_frame_get_popup_component (control_frame, &ev);
+	if (frame->priv->ui_component == NULL || BONOBO_EX (&ev)) {
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
+		CORBA_exception_free (&ev);
+		g_warning (G_STRLOC ": failed to load applet %s (can't get popup component)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
 		return NULL;
+	}
 
 	bonobo_ui_util_set_ui (frame->priv->ui_component, DATADIR,
 			       "GNOME_Panel_Popup.xml", "panel", NULL);
@@ -1132,11 +1286,48 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 		frame->priv->ui_component, popup_verbs, frame);
 
 	control = bonobo_control_frame_get_control (control_frame);
-	if (!control)
+	if (!control) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get control)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
 		return NULL;
+	}
 
 	frame->priv->applet_shell = panel_applet_frame_get_applet_shell (control);
+	if (frame->priv->applet_shell == CORBA_OBJECT_NIL) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get applet shell)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
 
+	CORBA_exception_free (&ev);
+
+	cnx_status = ORBit_small_get_connection_status (control);
+	if (cnx_status != ORBIT_CONNECTION_IN_PROC) {
+		ORBitConnection *cnx = ORBit_small_get_connection (control);
+
+		if (cnx == NULL) {
+			CORBA_exception_free (&ev);
+			panel_applet_frame_loading_failed (
+				frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+			g_warning (G_STRLOC ": failed to load applet %s (lost connection)", iid);
+			gtk_object_sink (GTK_OBJECT (widget));
+			return NULL;
+		}
+
+		g_signal_connect_object (
+			cnx,
+			"broken",
+			G_CALLBACK (panel_applet_frame_cnx_broken),
+			frame,
+			G_CONNECT_SWAPPED);
+	}
+	
 	gtk_container_add (GTK_CONTAINER (frame), widget);
 
 	return widget;
@@ -1153,10 +1344,8 @@ panel_applet_frame_new (PanelWidget *panel,
 
 	frame = g_object_new (PANEL_TYPE_APPLET_FRAME, NULL);
 
-	if (!panel_applet_frame_construct (frame, panel, iid, id)) {
-		gtk_object_sink (GTK_OBJECT (frame));
+	if (!panel_applet_frame_construct (frame, panel, iid, id))
 		return NULL;
-	}
 
 	return GTK_WIDGET (frame);
 }
