@@ -37,6 +37,8 @@
 #include "panel-widget.h"
 #include "panel-util.h"
 #include "panel-multiscreen.h"
+#include "panel-toplevel.h"
+#include "panel-lockdown.h"
 
 typedef struct {
 	GdkScreen       *screen;
@@ -323,7 +325,7 @@ panel_profile_set_commit_timeout (PanelToplevel *toplevel,
 			    GUINT_TO_POINTER (timeout));
 }
 
-static const guint
+static guint
 panel_profile_get_commit_timeout (GObject *object)
 {
 	if (!commit_timeout_quark)
@@ -1789,6 +1791,7 @@ panel_profile_load_object (GConfClient       *client,
 	type_string = gconf_client_get_string (client, key, NULL);
         
 	if (!panel_profile_map_object_type_string (type_string, &object_type)) {
+		g_free (id);
 		g_free (type_string);
 		return;
 	}
@@ -1851,13 +1854,20 @@ panel_profile_delete_dir (GConfClient       *client,
 		break;
 	}
 
+	if (type == PANEL_GCONF_TOPLEVELS) {
+		key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/%s/%s/background",
+					   current_profile, type_str, id);
+		gconf_client_remove_dir (client, key, NULL);
+	}
+
 	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/%s/%s",
 				   current_profile, type_str, id);
+	gconf_client_remove_dir (client, key, NULL);
 
-	if (type == PANEL_GCONF_TOPLEVELS)
-	    gconf_client_remove_dir (client, key, NULL);
-
-	panel_gconf_clean_dir (client, key);
+	gconf_client_recursive_unset (client,
+				      key,
+				      GCONF_UNSET_INCLUDING_SCHEMA_NAMES,
+				      NULL);
 }
 
 static gboolean
@@ -2103,69 +2113,6 @@ panel_profile_load_list (GConfClient           *client,
 	g_slist_free (list);
 }
 
-static void
-panel_profile_apply_schemas (GConfClient *client,
-			     const char  *profile_dir,
-			     const char  *schemas_dir,
-			     const char  *id_list_key,
-			     const char  *type)
-{
-
-	const char *key;
-	GSList     *list, *l;
-	GError     *error = NULL;
-
-	key = panel_gconf_sprintf ("%s/general/%s", profile_dir, id_list_key);
-	list = gconf_client_get_list (client, key, GCONF_VALUE_STRING, &error);
-	if (error) {
-		g_warning (_("Error reading GConf list value '%s': %s"),
-			   key, error->message);
-		g_error_free (error);
-		return;
-	}
-
-	for (l = list; l; l = l->next) {
-		char *dir;
-		char *id = l->data;
-
-		dir = g_strdup_printf ("%s/%s/%s", profile_dir, type, id);
-		panel_gconf_associate_schemas_in_dir (client, dir, schemas_dir);
-		g_free (dir);
-		g_free (id);
-	}
-
-	g_slist_free (list);
-}
-
-static void
-panel_profile_copy_defaults (GConfClient *client,
-			     const char  *profile_dir)
-{
-	char *dir;
-
-	panel_gconf_copy_dir (client, PANEL_DEFAULTS_DIR, profile_dir);
-
-	dir = gconf_concat_dir_and_key (profile_dir, "general");
-	panel_gconf_associate_schemas_in_dir (client, dir, PANEL_SCHEMAS_DIR "/general");
-	g_free (dir);
-
-	panel_profile_apply_schemas (client,
-				     profile_dir,
-				     PANEL_SCHEMAS_DIR "/toplevels",
-				     "toplevel_id_list",
-				     "toplevels");
-	panel_profile_apply_schemas (client,
-				     profile_dir,
-				     PANEL_SCHEMAS_DIR "/objects",
-				     "applet_id_list",
-				     "applets");
-	panel_profile_apply_schemas (client,
-				     profile_dir,
-				     PANEL_SCHEMAS_DIR "/objects",
-				     "object_id_list",
-				     "objects");
-}
-
 static GSList *
 panel_profile_copy_defaults_for_screen (GConfClient       *client,
 					const char        *profile_dir,
@@ -2206,7 +2153,6 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 		char *new_id;
 		char *src_dir;
 		char *dest_dir;
-		char *schema_key;
 
 		new_id = g_strdup_printf ("%s_screen%d", default_id, screen_n);
 
@@ -2215,11 +2161,6 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 
 		panel_gconf_copy_dir (client, src_dir, dest_dir);
 
-		schema_key = g_strdup_printf (PANEL_SCHEMAS_DIR "/%s", 
-			                      schemas_str);
-		panel_gconf_associate_schemas_in_dir (client, dest_dir, schema_key);
-		g_free (schema_key);
-
 		new_ids = g_slist_prepend (new_ids, new_id);
 
 		g_free (src_dir);
@@ -2403,7 +2344,7 @@ panel_profile_load (char *profile_name)
 
 	dir = gconf_concat_dir_and_key (PANEL_CONFIG_DIR, current_profile);
 	if (!gconf_client_dir_exists (client, dir, NULL))
-		panel_profile_copy_defaults (client, dir);
+		panel_gconf_copy_dir (client, PANEL_DEFAULTS_DIR, dir);
 
 	panel_compatibility_migrate_panel_id_list (client);
 
@@ -2484,28 +2425,24 @@ panel_profile_is_writable_show_program_list (void)
 	return gconf_client_key_is_writable (client, key, NULL);
 }
 
-/* True if all the move keys are writable.  This is kind of
-   a blanket statement.  For example we may only lock one of them
-   down and that would prevent the user from moving the panel at
-   all, but really, making this truly fine grained is probably too
-   much crack */
 gboolean
 panel_profile_can_be_moved_freely (PanelToplevel *toplevel)
 {
 	const char *key;
 	GConfClient *client;
-	
-	if ( ! panel_profile_is_writable_toplevel_orientation (toplevel))
+
+	if (panel_lockdown_get_locked_down () ||
+	    !panel_profile_is_writable_toplevel_orientation (toplevel))
 		return FALSE;
 
 	client = panel_gconf_get_client ();
 
 	key = panel_profile_get_toplevel_key (toplevel, "screen");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
 	key = panel_profile_get_toplevel_key (toplevel, "monitor");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
 	/* For expanded panels we don't really have to check 
@@ -2514,17 +2451,17 @@ panel_profile_can_be_moved_freely (PanelToplevel *toplevel)
 		return TRUE;
 
 	key = panel_profile_get_toplevel_key (toplevel, "x");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 	key = panel_profile_get_toplevel_key (toplevel, "x_centered");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
 	key = panel_profile_get_toplevel_key (toplevel, "y");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 	key = panel_profile_get_toplevel_key (toplevel, "y_centered");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
 	return TRUE;
